@@ -4,6 +4,8 @@
 Exception definitions.
 """
 
+from keystoneclient.openstack.common import jsonutils
+
 
 class CommandError(Exception):
     pass
@@ -19,7 +21,8 @@ class AuthorizationFailure(Exception):
 
 class NoTokenLookupException(Exception):
     """This form of authentication does not support looking up
-       endpoints from an existing token."""
+       endpoints from an existing token.
+    """
     pass
 
 
@@ -29,19 +32,18 @@ class EndpointNotFound(Exception):
 
 
 class EmptyCatalog(Exception):
-    """The service catalog is empty. """
+    """The service catalog is empty."""
     pass
 
 
 class NoUniqueMatch(Exception):
-    """Unable to find unique resource"""
+    """Unable to find unique resource."""
     pass
 
 
 class ClientException(Exception):
-    """
-    The base exception class for all exceptions this library raises.
-    """
+    """The base exception class for all exceptions this library raises."""
+
     def __init__(self, code, message=None, details=None):
         self.code = code
         self.message = message or self.__class__.message
@@ -52,24 +54,21 @@ class ClientException(Exception):
 
 
 class BadRequest(ClientException):
-    """
-    HTTP 400 - Bad request: you sent some malformed data.
-    """
+    """HTTP 400 - Bad request: you sent some malformed data."""
+
     http_status = 400
     message = "Bad request"
 
 
 class Unauthorized(ClientException):
-    """
-    HTTP 401 - Unauthorized: bad credentials.
-    """
+    """HTTP 401 - Unauthorized: bad credentials."""
+
     http_status = 401
     message = "Unauthorized"
 
 
 class Forbidden(ClientException):
-    """
-    HTTP 403 - Forbidden: your credentials don't give you access to this
+    """HTTP 403 - Forbidden: your credentials do not allow access to this
     resource.
     """
     http_status = 403
@@ -77,32 +76,26 @@ class Forbidden(ClientException):
 
 
 class NotFound(ClientException):
-    """
-    HTTP 404 - Not found
-    """
+    """HTTP 404 - Not found."""
     http_status = 404
     message = "Not found"
 
 
 class MethodNotAllowed(ClientException):
-    """
-    HTTP 405 - Method not allowed
-    """
+    """HTTP 405 - Method not allowed."""
     http_status = 405
     message = "Method not allowed"
 
 
 class Conflict(ClientException):
-    """
-    HTTP 409 - Conflict
-    """
+    """HTTP 409 - Conflict."""
     http_status = 409
     message = "Conflict"
 
 
 class OverLimit(ClientException):
-    """
-    HTTP 413 - Over limit: you're over the API limits for this time period.
+    """HTTP 413 - Over limit: you're over the API limits for this time
+    period.
     """
     http_status = 413
     message = "Over limit"
@@ -110,17 +103,15 @@ class OverLimit(ClientException):
 
 # NotImplemented is a python keyword.
 class HTTPNotImplemented(ClientException):
-    """
-    HTTP 501 - Not Implemented: the server does not support this operation.
+    """HTTP 501 - Not Implemented: the server does not support this
+    operation.
     """
     http_status = 501
     message = "Not Implemented"
 
 
 class ServiceUnavailable(ClientException):
-    """
-    HTTP 503 - Service Unavailable: The server is currently unavailable.
-    """
+    """HTTP 503 - Service Unavailable: The server is currently unavailable."""
     http_status = 503
     message = "Service Unavailable"
 
@@ -142,10 +133,9 @@ _code_map = dict((c.http_status, c) for c in [BadRequest,
                                               ServiceUnavailable])
 
 
-def from_response(response, body):
-    """
-    Return an instance of an ClientException or subclass
-    based on an requests response.
+def from_response(response, body=None):
+    """Return an instance of a ClientException or subclass
+    based on a requests response.
 
     Usage::
 
@@ -154,6 +144,12 @@ def from_response(response, body):
             raise exception_from_response(resp, resp.text)
     """
     cls = _code_map.get(response.status_code, ClientException)
+    if body is None:
+        try:
+            body = jsonutils.loads(response.text)
+        except Exception:
+            body = response.text
+
     if body:
         if hasattr(body, 'keys'):
             error = body[body.keys()[0]]
