@@ -14,8 +14,8 @@
 #    under the License.
 import logging
 
-from keystoneclient import client
 from keystoneclient import exceptions
+from keystoneclient import httpclient
 from keystoneclient.v2_0 import ec2
 from keystoneclient.v2_0 import endpoints
 from keystoneclient.v2_0 import roles
@@ -28,7 +28,7 @@ from keystoneclient.v2_0 import users
 _logger = logging.getLogger(__name__)
 
 
-class Client(client.HTTPClient):
+class Client(httpclient.HTTPClient):
     """Client for the OpenStack Keystone v2.0 API.
 
     :param string username: Username for authentication. (optional)
@@ -122,7 +122,7 @@ class Client(client.HTTPClient):
     """
 
     def __init__(self, **kwargs):
-        """Initialize a new client for the Keystone v2.0 API. """
+        """Initialize a new client for the Keystone v2.0 API."""
         super(Client, self).__init__(**kwargs)
         self.version = 'v2.0'
         self.endpoints = endpoints.EndpointManager(self)
@@ -138,33 +138,6 @@ class Client(client.HTTPClient):
         if self.management_url is None:
             self.authenticate()
 
-    def process_token(self):
-        """Extract and process information from the new auth_ref.
-
-        And set the relevant authentication information.
-        """
-        # if we got a response without a service catalog, set the local
-        # list of tenants for introspection, and leave to client user
-        # to determine what to do. Otherwise, load up the service catalog
-        if self.auth_ref.project_scoped:
-            if not self.auth_ref.tenant_id:
-                raise exceptions.AuthorizationFailure(
-                    "Token didn't provide tenant_id")
-            if self.management_url is None and self.auth_ref.management_url:
-                self.management_url = self.auth_ref.management_url[0]
-            self.project_name = self.auth_ref.tenant_name
-            self.project_id = self.auth_ref.tenant_id
-
-        if not self.auth_ref.user_id:
-            raise exceptions.AuthorizationFailure(
-                "Token didn't provide user_id")
-
-        self.user_id = self.auth_ref.user_id
-
-        self.auth_domain_id = self.auth_ref.domain_id
-        self.auth_tenant_id = self.auth_ref.tenant_id
-        self.auth_user_id = self.auth_ref.user_id
-
     def get_raw_token_from_identity_service(self, auth_url, username=None,
                                             password=None, tenant_name=None,
                                             tenant_id=None, token=None,
@@ -197,7 +170,8 @@ class Client(client.HTTPClient):
         """Takes a username, password, and optionally a tenant_id or
         tenant_name to get an authentication token from keystone.
         May also take a token and a tenant_id to re-scope a token
-        to a tenant."""
+        to a tenant.
+        """
         headers = {}
         url = auth_url + "/tokens"
         if token:
