@@ -19,8 +19,10 @@ import argparse
 import getpass
 import sys
 
-from keystoneclient.v2_0 import client
+import six
+
 from keystoneclient import utils
+from keystoneclient.v2_0 import client
 
 
 CLIENT_CLASS = client.Client
@@ -42,12 +44,16 @@ def require_service_catalog(f):
     return wrapped
 
 
-@utils.arg('--tenant-id', metavar='<tenant-id>',
-           help='Tenant ID;  lists all users if not specified')
+@utils.arg('--tenant', '--tenant-id', metavar='<tenant>',
+           help='Tenant;  lists all users if not specified')
 @utils.arg('--tenant_id', help=argparse.SUPPRESS)
 def do_user_list(kc, args):
-    """List users"""
-    users = kc.users.list(tenant_id=args.tenant_id)
+    """List users."""
+    if args.tenant:
+        tenant_id = utils.find_resource(kc.tenants, args.tenant).id
+    else:
+        tenant_id = None
+    users = kc.users.list(tenant_id=tenant_id)
     utils.print_list(users, ['id', 'name', 'enabled', 'email'],
                      order_by='name')
 
@@ -61,7 +67,7 @@ def do_user_get(kc, args):
 
 @utils.arg('--name', metavar='<user-name>', required=True,
            help='New user name (must be unique)')
-@utils.arg('--tenant-id', metavar='<tenant-id>',
+@utils.arg('--tenant', '--tenant-id', metavar='<tenant>',
            help='New user default tenant')
 @utils.arg('--tenant_id', help=argparse.SUPPRESS)
 @utils.arg('--pass', metavar='<pass>', dest='passwd',
@@ -72,8 +78,14 @@ def do_user_get(kc, args):
            help='Initial user enabled status (default true)')
 def do_user_create(kc, args):
     """Create new user"""
+    if args.tenant:
+        tenant_id = utils.find_resource(kc.tenants, args.tenant).id
+    elif args.tenant_id:
+        tenant_id = args.tenant_id
+    else:
+        tenant_id = None
     user = kc.users.create(args.name, args.passwd, args.email,
-                           tenant_id=args.tenant_id,
+                           tenant_id=tenant_id,
                            enabled=utils.string_to_bool(args.enabled))
     utils.print_dict(user._info)
 
@@ -86,7 +98,7 @@ def do_user_create(kc, args):
            help='Enable or disable user')
 @utils.arg('user', metavar='<user>', help='Name or ID of user to update')
 def do_user_update(kc, args):
-    """Update user's name, email, and enabled status"""
+    """Update user's name, email, and enabled status."""
     kwargs = {}
     if args.name:
         kwargs['name'] = args.name
@@ -112,7 +124,7 @@ def do_user_update(kc, args):
 @utils.arg('user', metavar='<user>',
            help='Name or ID of user to update password')
 def do_user_password_update(kc, args):
-    """Update user password"""
+    """Update user password."""
     user = utils.find_resource(kc.users, args.user)
     new_passwd = args.passwd or utils.prompt_for_password()
     if new_passwd is None:
@@ -129,7 +141,7 @@ def do_user_password_update(kc, args):
 @utils.arg('--new-password ', metavar='<new-password>', dest='newpasswd',
            required=False, help='Desired new password')
 def do_password_update(kc, args):
-    """Update own password"""
+    """Update own password."""
 
     # we are prompting for these passwords if they are not passed in
     # this gives users the option not to have their password
@@ -162,7 +174,7 @@ def do_user_delete(kc, args):
 
 
 def do_tenant_list(kc, args):
-    """List all tenants"""
+    """List all tenants."""
     tenants = kc.tenants.list()
     utils.print_list(tenants, ['id', 'name', 'enabled'], order_by='name')
 
@@ -170,7 +182,7 @@ def do_tenant_list(kc, args):
 @utils.arg('tenant', metavar='<tenant>',
            help='Name or ID of tenant to display')
 def do_tenant_get(kc, args):
-    """Display tenant details"""
+    """Display tenant details."""
     tenant = utils.find_resource(kc.tenants, args.tenant)
     utils.print_dict(tenant._info)
 
@@ -182,7 +194,7 @@ def do_tenant_get(kc, args):
 @utils.arg('--enabled', metavar='<true|false>', default=True,
            help='Initial tenant enabled status (default true)')
 def do_tenant_create(kc, args):
-    """Create new tenant"""
+    """Create new tenant."""
     tenant = kc.tenants.create(args.name,
                                description=args.description,
                                enabled=utils.string_to_bool(args.enabled))
@@ -197,7 +209,7 @@ def do_tenant_create(kc, args):
            help='Enable or disable tenant')
 @utils.arg('tenant', metavar='<tenant>', help='Name or ID of tenant to update')
 def do_tenant_update(kc, args):
-    """Update tenant name, description, enabled status"""
+    """Update tenant name, description, enabled status."""
     tenant = utils.find_resource(kc.tenants, args.tenant)
     kwargs = {}
     if args.name:
@@ -215,7 +227,7 @@ def do_tenant_update(kc, args):
 
 @utils.arg('tenant', metavar='<tenant>', help='Name or ID of tenant to delete')
 def do_tenant_delete(kc, args):
-    """Delete tenant"""
+    """Delete tenant."""
     tenant = utils.find_resource(kc.tenants, args.tenant)
     kc.tenants.delete(tenant)
 
@@ -228,7 +240,7 @@ def do_tenant_delete(kc, args):
 @utils.arg('--description', metavar='<service-description>',
            help='Description of service')
 def do_service_create(kc, args):
-    """Add service to Service Catalog"""
+    """Add service to Service Catalog."""
     service = kc.services.create(args.name,
                                  args.type,
                                  args.description)
@@ -236,7 +248,7 @@ def do_service_create(kc, args):
 
 
 def do_service_list(kc, args):
-    """List all services in Service Catalog"""
+    """List all services in Service Catalog."""
     services = kc.services.list()
     utils.print_list(services, ['id', 'name', 'type', 'description'],
                      order_by='name')
@@ -245,7 +257,7 @@ def do_service_list(kc, args):
 @utils.arg('service', metavar='<service>',
            help='Name or ID of service to display')
 def do_service_get(kc, args):
-    """Display service from Service Catalog"""
+    """Display service from Service Catalog."""
     service = utils.find_resource(kc.services, args.service)
     utils.print_dict(service._info)
 
@@ -253,20 +265,20 @@ def do_service_get(kc, args):
 @utils.arg('service', metavar='<service>',
            help='Name or ID of service to delete')
 def do_service_delete(kc, args):
-    """Delete service from Service Catalog"""
+    """Delete service from Service Catalog."""
     service = utils.find_resource(kc.services, args.service)
     kc.services.delete(service.id)
 
 
 def do_role_list(kc, args):
-    """List all roles"""
+    """List all roles."""
     roles = kc.roles.list()
     utils.print_list(roles, ['id', 'name'], order_by='name')
 
 
 @utils.arg('role', metavar='<role>', help='Name or ID of role to display')
 def do_role_get(kc, args):
-    """Display role details"""
+    """Display role details."""
     role = utils.find_resource(kc.roles, args.role)
     utils.print_dict(role._info)
 
@@ -274,14 +286,14 @@ def do_role_get(kc, args):
 @utils.arg('--name', metavar='<role-name>', required=True,
            help='Name of new role')
 def do_role_create(kc, args):
-    """Create new role"""
+    """Create new role."""
     role = kc.roles.create(args.name)
     utils.print_dict(role._info)
 
 
 @utils.arg('role', metavar='<role>', help='Name or ID of role to delete')
 def do_role_delete(kc, args):
-    """Delete role"""
+    """Delete role."""
     role = utils.find_resource(kc.roles, args.role)
     kc.roles.delete(role)
 
@@ -365,7 +377,7 @@ def do_user_role_list(kc, args):
 @utils.arg('--tenant-id', metavar='<tenant-id>', help='Tenant ID')
 @utils.arg('--tenant_id', help=argparse.SUPPRESS)
 def do_ec2_credentials_create(kc, args):
-    """Create EC2-compatible credentials for user per tenant"""
+    """Create EC2-compatible credentials for user per tenant."""
     if not args.tenant_id:
         # use the authenticated tenant id as a default
         args.tenant_id = kc.auth_tenant_id
@@ -381,7 +393,7 @@ def do_ec2_credentials_create(kc, args):
 @utils.arg('--access', metavar='<access-key>', required=True,
            help='Access Key')
 def do_ec2_credentials_get(kc, args):
-    """Display EC2-compatible credentials"""
+    """Display EC2-compatible credentials."""
     if not args.user_id:
         # use the authenticated user id as a default
         args.user_id = kc.auth_user_id
@@ -413,7 +425,7 @@ def do_ec2_credentials_list(kc, args):
 @utils.arg('--access', metavar='<access-key>', required=True,
            help='Access Key')
 def do_ec2_credentials_delete(kc, args):
-    """Delete EC2-compatible credentials"""
+    """Delete EC2-compatible credentials."""
     if not args.user_id:
         # use the authenticated user id as a default
         args.user_id = kc.auth_user_id
@@ -430,7 +442,7 @@ def do_ec2_credentials_delete(kc, args):
 def do_catalog(kc, args):
     """List service catalog, possibly filtered by service."""
     endpoints = kc.service_catalog.get_endpoints(service_type=args.service)
-    for (service, service_endpoints) in endpoints.iteritems():
+    for (service, service_endpoints) in six.iteritems(endpoints):
         if len(service_endpoints) > 0:
             print("Service: %s" % service)
             for ep in service_endpoints:
@@ -449,7 +461,7 @@ def do_catalog(kc, args):
            help='Value of attribute to match')
 @require_service_catalog
 def do_endpoint_get(kc, args):
-    """Find endpoint filtered by a specific attribute or service type"""
+    """Find endpoint filtered by a specific attribute or service type."""
     kwargs = {
         'service_type': args.service,
         'endpoint_type': args.endpoint_type,
@@ -466,7 +478,7 @@ def do_endpoint_get(kc, args):
 
 
 def do_endpoint_list(kc, args):
-    """List configured service endpoints"""
+    """List configured service endpoints."""
     endpoints = kc.endpoints.list()
     utils.print_list(endpoints,
                      ['id', 'region', 'publicurl',
@@ -475,8 +487,9 @@ def do_endpoint_list(kc, args):
 
 @utils.arg('--region', metavar='<endpoint-region>',
            help='Endpoint region', default='regionOne')
-@utils.arg('--service-id', '--service_id', metavar='<service-id>',
-           required=True, help='ID of service associated with Endpoint')
+@utils.arg('--service', '--service-id', '--service_id',
+           metavar='<service>', required=True,
+           help='Name or ID of service associated with Endpoint')
 @utils.arg('--publicurl', metavar='<public-url>',
            help='Public URL endpoint')
 @utils.arg('--adminurl', metavar='<admin-url>',
@@ -484,9 +497,10 @@ def do_endpoint_list(kc, args):
 @utils.arg('--internalurl', metavar='<internal-url>',
            help='Internal URL endpoint')
 def do_endpoint_create(kc, args):
-    """Create a new endpoint associated with a service"""
+    """Create a new endpoint associated with a service."""
+    service_id = utils.find_resource(kc.services, args.service).id
     endpoint = kc.endpoints.create(args.region,
-                                   args.service_id,
+                                   service_id,
                                    args.publicurl,
                                    args.adminurl,
                                    args.internalurl)
@@ -495,7 +509,7 @@ def do_endpoint_create(kc, args):
 
 @utils.arg('id', metavar='<endpoint-id>', help='ID of endpoint to delete')
 def do_endpoint_delete(kc, args):
-    """Delete a service endpoint"""
+    """Delete a service endpoint."""
     try:
         kc.endpoints.delete(args.id)
         print('Endpoint has been deleted.')
@@ -507,6 +521,6 @@ def do_endpoint_delete(kc, args):
            help='wrap PKI tokens to a specified length, or 0 to disable')
 @require_service_catalog
 def do_token_get(kc, args):
-    """Display the current user token"""
+    """Display the current user token."""
     utils.print_dict(kc.service_catalog.get_token(),
                      wrap=int(args.wrap))
